/*
 * Copyright (C) 2009 Chase Douglas
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.
 */

#include "Client.h"
#include "macros.h"

Client::Client(QString &hostname, quint16 port) :
        connection(hostname, port) {
    connect(&connection, SIGNAL(connected()), SLOT(connected()));
    connect(&connection, SIGNAL(encrypted()), SLOT(encrypted()));
    connect(&connection, SIGNAL(authenticated()), SLOT(authenticated()));
    connect(&connection, SIGNAL(readyRead()), SLOT(readyRead()));
    connect(&connection, SIGNAL(disconnected()), SLOT(disconnected()));
}

void Client::connected() {
#ifndef NO_ENCRYPTION
    connection.startEncryption();
#else
#ifndef NO_AUTHENTICATION
    connection.startAuthentication();
#else
    connection.startCommunication();
    emit ready();
#endif
#endif
}

void Client::encrypted() {
#ifndef NO_AUTHENTICATION
    connection.startAuthentication();
#else
    connection.startCommunication();
    emit ready();
#endif
}

void Client::authenticated() {
    connection.startCommunication();
    emit ready();
}

void Client::readyRead() {
    while (connection.bytesAvailable()) {
        quint8 len;
        if (connection.peek((char *)&len, 1) < 0) {
            qWarning("Warning: Failed to peek data from connection");
            return;
        }

        if (connection.bytesAvailable() < len + 1) {
            return;
        }

        QByteArray message(connection.read(len + 1));
        message.remove(0, 1);
        qDebug("Read '%s' from server", message.data());

        if (message == "CLOSE") {
            connection.disconnect();
            return;
        }
    }
}

bool Client::write(QByteArray message) {
    return connection.write(message);
}

void Client::disconnected() {
    emit quit();
}
